package cn.com.infosec.mobile.android.framework.crypto.impl.gm;

import cn.com.infosec.asn1.ASN1EncodableVector;
import cn.com.infosec.asn1.ASN1InputStream;
import cn.com.infosec.asn1.ASN1Sequence;
import cn.com.infosec.asn1.DERInteger;
import cn.com.infosec.asn1.DEROctetString;
import cn.com.infosec.asn1.DERSequence;
import cn.com.infosec.math.ec.ECPoint;
import cn.com.infosec.util.BigIntegers;
import java.io.IOException;
import java.math.BigInteger;
import java.nio.ByteBuffer;
import java.security.SecureRandom;
import java.util.Arrays;

public class SM2Impl {
   public static final int SM2_SIZE = 32;
   private static final ECPoint gmg;
   private static final BigInteger gmn;
   private static final SecureRandom random;

   private static SecureRandom getRandom() {
      SecureRandom var10000;
      SecureRandom var0;
      try {
         var10000 = SecureRandom.getInstance("SHA1PRNG");
      } catch (Exception var1) {
         var0 = new SecureRandom.<init>();
         return var0;
      }

      var0 = var10000;
      return var0;
   }

   private static ECPoint kmg(ECPoint p, BigInteger k) {
      return p.multiply(k);
   }

   public static void genKeyPair(byte[] privKey, byte[] pubKeyX, byte[] pubKeyY) {
      int var3 = gmn.bitLength();

      BigInteger var4;
      ECPoint var5;
      do {
         var4 = (new BigInteger(var3, random)).mod(gmn);
      } while((var5 = gmg.multiply(var4)).isInfinity());

      for(var3 = 0; var3 < 32; ++var3) {
         privKey[var3] = 0;
         pubKeyX[var3] = 0;
         pubKeyY[var3] = 0;
      }

      byte[] var10002 = BigIntegers.asUnsignedByteArray(var4);
      var3 = 32 - var10002.length;
      int var8 = var10002.length;
      System.arraycopy(var10002, 0, privKey, var3, var8);
      byte[] var10001 = BigIntegers.asUnsignedByteArray(var5.getX().toBigInteger());
      int privKey1 = 32 - var10001.length;
      var3 = var10001.length;
      System.arraycopy(var10001, 0, pubKeyX, privKey1, var3);
      byte[] var10000 = BigIntegers.asUnsignedByteArray(var5.getY().toBigInteger());
      privKey1 = 32 - var10000.length;
      int pubKeyX1 = var10000.length;
      System.arraycopy(var10000, 0, pubKeyY, privKey1, pubKeyX1);
   }

   public static void genKeyPair(byte[] privkey, byte[] pubkey) {
      int var2 = gmn.bitLength();

      BigInteger var3;
      ECPoint var4;
      do {
         var3 = (new BigInteger(var2, random)).mod(gmn);
      } while((var4 = gmg.multiply(var3)).isInfinity());

      byte[] var6 = BigIntegers.asUnsignedByteArray(var3);

      int var7;
      for(var7 = 0; var7 < 32; ++var7) {
         privkey[var7] = 0;
      }

      var2 = 32 - var6.length;
      var7 = var6.length;
      System.arraycopy(var6, 0, privkey, var2, var7);
      pubkey[0] = 4;

      int privkey1;
      for(privkey1 = 1; privkey1 < 65; ++privkey1) {
         pubkey[privkey1] = 0;
      }

      byte[] var10001 = BigIntegers.asUnsignedByteArray(var4.getX().toBigInteger());
      privkey1 = 33 - var10001.length;
      var2 = var10001.length;
      System.arraycopy(var10001, 0, pubkey, privkey1, var2);
      byte[] var10000 = BigIntegers.asUnsignedByteArray(var4.getY().toBigInteger());
      privkey1 = 65 - var10000.length;
      var2 = var10000.length;
      System.arraycopy(var10000, 0, pubkey, privkey1, var2);
   }

   public static byte[] signHash(byte[] msg_hash, byte[] privkey) {
      if (msg_hash.length == 32 && privkey.length == 32) {
         BigInteger var2;
         var2 = new BigInteger.<init>(1, msg_hash);

         BigInteger var3;
         BigInteger var4;
         BigInteger var5;
         BigInteger var6;
         BigInteger msg_hash2;
         do {
            int msg_hash1 = gmn.bitLength();

            BigInteger var10000;
            do {
               var10000 = var3 = new BigInteger;
               var10000.<init>(msg_hash1, random);
            } while(var10000.equals(var4 = BigInteger.ZERO) || var3.compareTo(var5 = gmn) >= 0 || (var6 = kmg(gmg, var3).getX().toBigInteger().mod(var5)).equals(var4) || var2.add(var6).compareTo(var5) == 0);

            msg_hash2 = new BigInteger.<init>(1, privkey);
            var6 = var2.add(var6).mod(var5);
         } while((msg_hash2 = BigInteger.ONE.add(msg_hash2).modInverse(var5).multiply(var3.add(msg_hash2.multiply(var6).negate())).mod(var5)).equals(var4));

         return xy2sign(BigIntegers.asUnsignedByteArray(var6), BigIntegers.asUnsignedByteArray(msg_hash2));
      } else {
         return null;
      }
   }

   public static byte[] signHashWithoutASN(byte[] msg_hash, byte[] privkey) {
      if (msg_hash.length == 32 && privkey.length == 32) {
         BigInteger var2;
         var2 = new BigInteger.<init>(1, msg_hash);

         BigInteger var3;
         BigInteger var4;
         BigInteger var5;
         BigInteger var6;
         int msg_hash1;
         BigInteger msg_hash2;
         BigInteger var10000;
         do {
            msg_hash1 = gmn.bitLength();

            do {
               var10000 = var3 = new BigInteger;
               var10000.<init>(msg_hash1, random);
            } while(var10000.equals(var4 = BigInteger.ZERO) || var3.compareTo(var5 = gmn) >= 0 || (var6 = kmg(gmg, var3).getX().toBigInteger().mod(var5)).equals(var4) || var2.add(var6).compareTo(var5) == 0);

            msg_hash2 = new BigInteger.<init>(1, privkey);
            var6 = var2.add(var6).mod(var5);
         } while((msg_hash2 = BigInteger.ONE.add(msg_hash2).modInverse(var5).multiply(var3.add(msg_hash2.multiply(var6).negate())).mod(var5)).equals(var4));

         var10000 = msg_hash2;
         msg_hash = BigIntegers.asUnsignedByteArray(var6);
         privkey = BigIntegers.asUnsignedByteArray(var10000);
         byte[] var9;
         byte[] var10 = var9 = new byte[64];
         byte[] var10003 = msg_hash;
         msg_hash1 = msg_hash.length;
         System.arraycopy(var10003, 0, var9, 0, msg_hash1);
         msg_hash1 = privkey.length;
         System.arraycopy(privkey, 0, var9, 32, msg_hash1);
         return var10;
      } else {
         return null;
      }
   }

   public static boolean verifyHash(byte[] msg_hash, byte[] r, byte[] s, byte[] x, byte[] y) {
      byte[] var10000 = msg_hash;
      msg_hash = new byte[64];
      System.arraycopy(r, 0, msg_hash, 0, 32);
      System.arraycopy(s, 0, msg_hash, 32, 32);
      (r = new byte[65])[0] = 4;
      System.arraycopy(x, 0, r, 1, 32);
      System.arraycopy(y, 0, r, 33, 32);
      return verifyHash(var10000, msg_hash, r);
   }

   public static boolean verifyHash(byte[] msg_hash, byte[] signature, byte[] pubkey) {
      if (msg_hash.length != 32) {
         return false;
      } else if (signature[1] != signature[3] + signature[signature[3] + 5] + 4) {
         return false;
      } else if (pubkey.length != 64 && pubkey.length != 65) {
         return false;
      } else {
         byte[] var10000 = signature;
         BigInteger signature1;
         signature1 = new BigInteger.<init>(1, msg_hash);
         byte[] var3;
         sign2xy(var10000, msg_hash = new byte[32], var3 = new byte[32]);
         BigInteger var4;
         BigInteger var10 = var4 = new BigInteger;
         var10.<init>(1, msg_hash);
         BigInteger msg_hash1;
         msg_hash1 = new BigInteger.<init>(1, var3);
         BigInteger var5;
         BigInteger var9;
         if (var10.compareTo(var9 = BigInteger.ONE) >= 0 && var4.compareTo(var5 = gmn) < 0) {
            if (msg_hash1.compareTo(var9) >= 0 && msg_hash1.compareTo(var5) < 0) {
               var10000 = pubkey;
               BigInteger pubkey1 = var4.add(msg_hash1).mod(var5);
               return gmg.multiply(msg_hash1).add(SM2Util.octect2point(var10000).multiply(pubkey1)).getX().toBigInteger().add(signature1).mod(var5).compareTo(var4) == 0;
            } else {
               return false;
            }
         } else {
            return false;
         }
      }
   }

   public static boolean verifyHashWithoutASN(byte[] msg_hash, byte[] signature, byte[] pubkey) {
      if (msg_hash.length != 32) {
         return false;
      } else if (pubkey.length != 64 && pubkey.length != 65) {
         return false;
      } else {
         BigInteger var3;
         var3 = new BigInteger.<init>(1, msg_hash);
         BigInteger msg_hash1;
         BigInteger var10000 = msg_hash1 = new BigInteger;
         var10000.<init>(1, Arrays.copyOf(signature, 32));
         BigInteger var4;
         var4 = new BigInteger.<init>(1, Arrays.copyOfRange(signature, 32, signature.length));
         BigInteger var5;
         BigInteger signature1;
         if (var10000.compareTo(signature1 = BigInteger.ONE) >= 0 && msg_hash1.compareTo(var5 = gmn) < 0) {
            if (var4.compareTo(signature1) >= 0 && var4.compareTo(var5) < 0) {
               signature1 = msg_hash1.add(var4).mod(var5);
               return gmg.multiply(var4).add(SM2Util.octect2point(pubkey).multiply(signature1)).getX().toBigInteger().add(var3).mod(var5).compareTo(msg_hash1) == 0;
            } else {
               return false;
            }
         } else {
            return false;
         }
      }
   }

   public static byte[] encrypt(byte[] msg, byte[] pubkey) {
      ASN1EncodableVector var2;
      var2 = new ASN1EncodableVector.<init>();
      int var3 = gmn.bitLength();

      SecureRandom var4;
      label50: {
         SecureRandom var10000;
         try {
            var10000 = SecureRandom.getInstance("SHA1PRNG");
         } catch (Exception var8) {
            var4 = new SecureRandom.<init>();
            break label50;
         }

         var4 = var10000;
      }

      BigInteger var5;
      ECPoint var6;
      ECPoint var12;
      do {
         while((var5 = (new BigInteger(var3, var4)).mod(gmn)).compareTo(BigInteger.ZERO) <= 0) {
         }

         var6 = gmg.multiply(var5);
      } while((var12 = SM2Util.octect2point(pubkey).multiply(var5)).isInfinity());

      DERInteger var10;
      byte[] var10001;
      byte[] var19;
      if ((pubkey = BigIntegers.asUnsignedByteArray(var6.getX().toBigInteger()))[0] < 128 && pubkey[0] >= 0) {
         var10 = new DERInteger.<init>(pubkey);
      } else {
         var19 = pubkey;
         var10001 = pubkey;
         (pubkey = new byte[pubkey.length + 1])[0] = 0;
         var3 = var10001.length;
         System.arraycopy(var19, 0, pubkey, 1, var3);
         var10 = new DERInteger.<init>(pubkey);
      }

      DERInteger var13;
      if ((pubkey = BigIntegers.asUnsignedByteArray(var6.getY().toBigInteger()))[0] < 128 && pubkey[0] >= 0) {
         var13 = new DERInteger.<init>(pubkey);
      } else {
         var19 = pubkey;
         var10001 = pubkey;
         (pubkey = new byte[pubkey.length + 1])[0] = 0;
         int var11 = var10001.length;
         System.arraycopy(var19, 0, pubkey, 1, var11);
         var13 = new DERInteger.<init>(pubkey);
      }

      (pubkey = new byte[65])[0] = 4;
      byte[] var10002 = BigIntegers.asUnsignedByteArray(var12.getX().toBigInteger());
      int var14 = 33 - var10002.length;
      int var15 = var10002.length;
      System.arraycopy(var10002, 0, pubkey, var14, var15);
      var10001 = BigIntegers.asUnsignedByteArray(var12.getY().toBigInteger());
      var14 = 65 - var10001.length;
      var15 = var10001.length;
      System.arraycopy(var10001, 0, pubkey, var14, var15);
      byte[] var16 = new byte[32];
      SM3Digest var17;
      SM3Digest var21 = var17 = new SM3Digest;
      var17.<init>();
      var17.update(ByteBuffer.allocate(32).put(pubkey, 1, 32).array());
      var17.update(msg);
      var17.update(ByteBuffer.allocate(32).put(pubkey, 33, 32).array());
      var21.digest(var16);
      DEROctetString var18;
      var18 = new DEROctetString.<init>(var16);
      pubkey = kdf(ByteBuffer.allocate(64).put(pubkey, 1, 64).array(), msg.length);
      var16 = new byte[msg.length];

      for(int var7 = 0; var7 < msg.length; ++var7) {
         var16[var7] = (byte)(msg[var7] ^ pubkey[var7]);
      }

      DEROctetString msg1;
      msg1 = new DEROctetString.<init>(var16);
      var2.add(var10);
      var2.add(var13);
      var2.add(var18);
      var2.add(msg1);
      return (new DERSequence(var2)).getEncoded();
   }

   public static byte[] encryptWithoutASN(byte[] msg, byte[] pubkey) {
      new ASN1EncodableVector();
      int var2 = gmn.bitLength();

      SecureRandom var3;
      label30: {
         SecureRandom var10000;
         try {
            var10000 = SecureRandom.getInstance("SHA1PRNG");
         } catch (Exception var9) {
            var3 = new SecureRandom.<init>();
            break label30;
         }

         var3 = var10000;
      }

      BigInteger var4;
      ECPoint var5;
      ECPoint var13;
      do {
         while((var4 = (new BigInteger(var2, var3)).mod(gmn)).compareTo(BigInteger.ZERO) <= 0) {
         }

         var5 = gmg.multiply(var4);
      } while((var13 = SM2Util.octect2point(pubkey).multiply(var4)).isInfinity());

      pubkey = BigIntegers.asUnsignedByteArray(var5.getX().toBigInteger());
      byte[] var11 = BigIntegers.asUnsignedByteArray(var5.getY().toBigInteger());
      byte[] var12;
      (var12 = new byte[65])[0] = 4;
      byte[] var10002 = BigIntegers.asUnsignedByteArray(var13.getX().toBigInteger());
      int var14 = 33 - var10002.length;
      int var16 = var10002.length;
      System.arraycopy(var10002, 0, var12, var14, var16);
      byte[] var21 = BigIntegers.asUnsignedByteArray(var13.getY().toBigInteger());
      var14 = 65 - var21.length;
      var16 = var21.length;
      System.arraycopy(var21, 0, var12, var14, var16);
      byte var17 = 32;
      byte[] var18 = new byte[32];
      SM3Digest var6;
      SM3Digest var22 = var6 = new SM3Digest;
      var6.<init>();
      var6.update(ByteBuffer.allocate(32).put(var12, 1, 32).array());
      var6.update(msg);
      var6.update(ByteBuffer.allocate(32).put(var12, 33, 32).array());
      var22.digest(var18);
      var12 = kdf(ByteBuffer.allocate(64).put(var12, 1, 64).array(), msg.length);
      int var19;
      byte[] var7 = new byte[var19 = msg.length];

      for(int var8 = 0; var8 < msg.length; ++var8) {
         var7[var8] = (byte)(msg[var8] ^ var12[var8]);
      }

      byte[] var20 = msg = new byte[pubkey.length + var11.length + var17 + var19 + 1];
      msg[0] = 4;
      int pubkey1 = pubkey.length;
      System.arraycopy(pubkey, 0, msg, 1, pubkey1);
      pubkey1 = pubkey.length + 1;
      int var15 = var11.length;
      System.arraycopy(var11, 0, msg, pubkey1, var15);
      pubkey1 = pubkey.length + var11.length + 1;
      System.arraycopy(var18, 0, msg, pubkey1, var17);
      pubkey1 = pubkey.length + var11.length + var17 + 1;
      System.arraycopy(var7, 0, msg, pubkey1, var19);
      return var20;
   }

   public static byte[] decrypt(byte[] ct, byte[] privkey) {
      ASN1Sequence ct1;
      if ((ct1 = (ASN1Sequence)(new ASN1InputStream(ct)).readObject()).size() != 4) {
         return null;
      } else {
         ct = ((DERInteger)ct1.getObjectAt(0)).getValue().toByteArray();
         byte[] var2 = ((DERInteger)ct1.getObjectAt(1)).getValue().toByteArray();
         byte[] var3 = ((DEROctetString)ct1.getObjectAt(2)).getOctets();
         byte[] var4 = ((DEROctetString)ct1.getObjectAt(3)).getOctets();
         int var5 = var4.length;
         byte[] var6;
         (var6 = new byte[65])[0] = 4;
         int ct2;
         if (ct.length <= 32) {
            ct2 = 32 - ct.length + 1;
            int var7 = ct.length;
            System.arraycopy(ct, 0, var6, ct2, var7);
         } else {
            if (ct.length != 33 || ct[0] != 0) {
               throw new IOException("Invalid point X");
            }

            System.arraycopy(ct, 1, var6, 1, 32);
         }

         int var12;
         if (var2.length <= 32) {
            ct2 = 32 - var2.length + 33;
            var12 = var2.length;
            System.arraycopy(var2, 0, var6, ct2, var12);
         } else {
            if (var2.length != 33 || var2[0] != 0) {
               throw new IOException("Invalid point Y");
            }

            System.arraycopy(var2, 1, var6, 33, 32);
         }

         ECPoint ct3;
         privkey = BigIntegers.asUnsignedByteArray((ct3 = SM2Util.octect2point(var6).multiply(new BigInteger(1, privkey))).getX().toBigInteger());
         byte[] var16 = var2 = new byte[64];
         ct2 = 32 - privkey.length;
         int privkey1 = privkey.length;
         System.arraycopy(privkey, 0, var2, ct2, privkey1);
         byte[] var17 = BigIntegers.asUnsignedByteArray(ct3.getY().toBigInteger());
         ct2 = 64 - var17.length;
         privkey1 = var17.length;
         System.arraycopy(var17, 0, var2, ct2, privkey1);
         ct = kdf(var16, var5);
         privkey = new byte[var5];

         for(int var13 = 0; var13 < var5; ++var13) {
            privkey[var13] = (byte)(var4[var13] ^ ct[var13]);
         }

         SM3Digest var15 = new SM3Digest();
         var15.update(ByteBuffer.allocate(32).put(var2, 0, 32).array());
         var15.update(privkey);
         var15.update(ByteBuffer.allocate(32).put(var2, 32, 32).array());
         var15.digest(ct = new byte[32]);

         for(var12 = 0; var12 < 32; ++var12) {
            if ((ct[var12] & 255) != (var3[var12] & 255)) {
               return null;
            }
         }

         return privkey;
      }
   }

   public static byte[] decryptWithoutASN(byte[] ct, byte[] privkey) {
      if (ct.length < 97) {
         return null;
      } else {
         byte[] var10000 = ct;
         byte[] var10001 = ct;
         byte[] var10002 = ct;
         ct = Arrays.copyOf(ct, 65);
         byte[] var2 = Arrays.copyOfRange(var10002, 65, 97);
         byte[] var3;
         int var4;
         int var11 = var4 = (var3 = Arrays.copyOfRange(var10000, 97, var10001.length)).length;
         ECPoint ct1;
         privkey = BigIntegers.asUnsignedByteArray((ct1 = SM2Util.octect2point(ct).multiply(new BigInteger(1, privkey))).getX().toBigInteger());
         byte[] var5;
         var10001 = var5 = new byte[64];
         int ct2 = 32 - privkey.length;
         int privkey1 = privkey.length;
         System.arraycopy(privkey, 0, var5, ct2, privkey1);
         byte[] var13 = BigIntegers.asUnsignedByteArray(ct1.getY().toBigInteger());
         ct2 = 64 - var13.length;
         privkey1 = var13.length;
         System.arraycopy(var13, 0, var5, ct2, privkey1);
         ct = kdf(var10001, var4);
         privkey = new byte[var11];

         for(int var6 = 0; var6 < var4; ++var6) {
            privkey[var6] = (byte)(var3[var6] ^ ct[var6]);
         }

         SM3Digest var12 = new SM3Digest();
         var12.update(ByteBuffer.allocate(32).put(var5, 0, 32).array());
         var12.update(privkey);
         var12.update(ByteBuffer.allocate(32).put(var5, 32, 32).array());
         var12.digest(ct = new byte[32]);

         for(int var10 = 0; var10 < 32; ++var10) {
            if ((ct[var10] & 255) != (var2[var10] & 255)) {
               return null;
            }
         }

         return privkey;
      }
   }

   private static byte[] kdf(byte[] x2y2, int keySize) {
      if (keySize < 1) {
         return null;
      } else {
         SM3Digest var2;
         var2 = new SM3Digest.<init>();
         byte var3 = 32;
         int var10001 = keySize;
         int var10002 = keySize;
         keySize /= var3;
         int var4 = var10002 % var3;
         byte[] var5 = new byte[var10001];
         byte[] var6 = new byte[32];
         int var7 = 1;

         byte[] var10007;
         for(int var8 = 0; var8 < keySize; ++var7) {
            boolean var14;
            byte[] var15;
            SM3Digest var10000;
            byte var10003;
            SM3Digest var10004;
            SM3Digest var10005;
            byte[] var10006;
            try {
               var10000 = var2;
               var15 = var5;
               var10002 = var8;
               var10003 = var3;
               var10004 = var2;
               var10005 = var2;
               var10006 = x2y2;
               var10007 = ByteBuffer.allocate(4).putInt(var7).array();
            } catch (Exception var12) {
               var14 = false;
               throw new RuntimeException("internal error");
            }

            byte[] var9 = var10007;

            try {
               var10005.update(var10006);
               var10004.update(var9);
            } catch (Exception var11) {
               var14 = false;
               throw new RuntimeException("internal error");
            }

            var10002 *= var10003;

            try {
               var10000.digest(var15, var10002);
            } catch (Exception var10) {
               var14 = false;
               throw new RuntimeException("internal error");
            }

            ++var8;
         }

         if (var4 != 0) {
            var10007 = x2y2;
            x2y2 = ByteBuffer.allocate(4).putInt(var7).array();
            var2.update(var10007);
            var2.update(x2y2);
            var2.digest(var6);
            int x2y21 = keySize * var3;
            System.arraycopy(var6, 0, var5, x2y21, var4);
         }

         return var5;
      }
   }

   private static void sign2xy(byte[] sign, byte[] x, byte[] y) {
      byte var3;
      byte var10000 = var3 = sign[3];
      byte var4 = sign[var3 + 5];
      if (var10000 > 32) {
         System.arraycopy(sign, var3 + 4 - 32, x, 0, 32);
      } else {
         int var5 = 32 - var3;
         System.arraycopy(sign, 4, x, var5, var3);
      }

      if (var4 > 32) {
         System.arraycopy(sign, var3 + 6 + var4 - 32, y, 0, 32);
      } else {
         byte[] var8 = sign;
         int sign1 = var3 + 6;
         int x1 = 32 - var4;
         System.arraycopy(var8, sign1, y, x1, var4);
      }

   }

   private static byte[] xy2sign(byte[] x, byte[] y) {
      int var2 = 0;
      byte var3 = 0;
      if (x.length == 32 && (x[0] & 255) >= 128) {
         var2 = 1;
      }

      if (y.length == 32 && (y[0] & 255) >= 128) {
         var3 = 1;
      }

      byte[] var4;
      byte[] var10001 = var4 = new byte[var2 + 70 + var3];
      var4[0] = 48;
      var4[1] = (byte)(var2 + 68 + var3);
      var4[2] = 2;
      var4[var2 + 3] = 0;
      var10001[3] = (byte)(var2 + 32);
      int var5 = 32 - x.length;

      int var6;
      for(var6 = 0; var6 < var5; ++var6) {
         var4[var2 + 4 + var6] = 0;
      }

      for(var6 = 0; var6 < x.length; ++var6) {
         int var7 = var2 + 4 + var6 + var5;
         var4[var7] = x[var6];
      }

      var4[var2 + 36] = 2;
      int x1;
      var4[x1 = var2 + 38] = 0;
      var2 += 37;
      var4[var2] = (byte)(var3 + 32);
      var2 = 32 - y.length;

      for(var5 = 0; var5 < var2; ++var5) {
         var4[x1 + var3 + var5] = 0;
      }

      for(var5 = 0; var5 < y.length; ++var5) {
         var6 = x1 + var3 + var5 + var2;
         var4[var6] = y[var5];
      }

      return var4;
   }

   static {
      gmg = SM2GMParameter.gmg;
      gmn = SM2GMParameter.gmn;
      random = getRandom();
   }
}
